==> react-native-data-forms-2.0.6.tgz: package/index.tsx <==
/**
 * #later
 * Currently, testing this repo is very annoying because I have to go through NPM publish and download it on the other project. Normally, it should be possible to use `npm link` (see `http://matthewsessions.com/2017/10/27/developing-rn-module.html`), but it doesn't work because the Metro Bundler `doesn't support symlinks`. (Error I got: `Expected path {path} to be relative to one of the project roots`). A solution would be using `Haul` but until recently that didn't work with `Expo` unless you ejected your project. However, in this issue, the below comment states he got it working: `https://github.com/callstack/haul/issues/463`. Try this approach!
 * 
 * - [ ] Try haul for Dunbar: see dunbar `haul.js`
 * - [ ] Symlink both projects at the same time.
 * - [ ] See if I can update on the fly
 * 
 * 
 * 
 * #marketing
- [ ] There must be other libraries exposing a range of input-type-components. 
      Would be great if it was easy to add this to it. #idea
- [ ] Copy docs into article and publish to medium, dev.to, LinkedIn and Karsens #later
- [ ] Wait for stars a week or so, after a week, reach out to tutorial writers and agencies like Spencer, 
      NativeBase, RN elements, Awesome react native, native.directory, slack groups, spectrum chats, and many more. #later
 *  */

import React from "react";

import {
  Text,
  ActivityIndicator,
  Platform,
  ScrollView,
  KeyboardAvoidingView,
  View,
  Keyboard
} from "react-native";

import { DataFormProps } from "./types";
import {
  removeDuplicates,
  notNullNorUndefined,
  mergeObjectArray
} from "./utils";

import { C } from "./constants";
import Button from "./button.component";

type DataFormState = {
  loading: boolean;
};

type KASProps = {
  noNavigationBelow: boolean;
  footer: React.ReactElement; // dunno
};

/**
 * KAS = KeyboardAvoidingScrollView
 * #todo: externalize to `leckr-util` or so
 */
class KAS extends React.Component<KASProps> {
  render() {
    const { noNavigationBelow, footer, children } = this.props;

    const offsetAdded = noNavigationBelow && Platform.OS === "ios" ? 50 : 0;
    const defaultOffset = Platform.OS === "ios" ? 20 : 75;
    const totalOffset = defaultOffset + offsetAdded;

    return (
      <KeyboardAvoidingView
        style={{
          height: "100%",
          backgroundColor: "#DDD"
        }}
        behavior="padding"
        keyboardVerticalOffset={totalOffset}
      >
        <ScrollView
          keyboardShouldPersistTaps={"handled"}
          style={{
            backgroundColor: C.BG_COLOR
          }}
        >
          {children}
        </ScrollView>
        {footer}
      </KeyboardAvoidingView>
    );
  }
}

class DataForm extends React.Component<DataFormProps, DataFormState> {
  constructor(props: DataFormProps) {
    super(props);
    this.saveValues = this.saveValues.bind(this); //to give props
    this.state = { loading: false };
  }

  renderInput = ({ field, value, key }) => {
    const { inputTypes, FieldComponent } = this.props;

    const isHidden = !!(
      field.hidden && field.hidden(this.getAllCurrentValues())
    );

    const inputProps = {
      state: this.state,
      setFormState: newState => this.setState(newState),
      value,
      ...field
    };

    const InputClass = field.type
      ? inputTypes[field.type]
      : inputTypes[Object.keys(inputTypes)[0]];

    if (InputClass && !isHidden) {
      const inputField = <InputClass {...inputProps} />;
      return FieldComponent({
        inputField,
        inputProps: { ...field },
        state: this.state,
        key
      });
    }

    return null;
  };

  clearState = () => {
    const { fields } = this.props;
    const allFields = fields.reduce(
      (result, f) => ({ ...result, [f.field]: undefined }),
      {}
    );

    this.setState(allFields);
  };

  getAllCurrentValues() {
    const { fields, values } = this.props;

    const valueKeysFromFields = fields.map(field => field.field);
    const valueKeysFromValues = Object.keys(values);
    const reallyAllValueKeys = removeDuplicates(
      valueKeysFromFields.concat(valueKeysFromValues)
    );

    return (
      values &&
      mergeObjectArray(
        reallyAllValueKeys.map(valueKey => {
          return {
            [valueKey]: notNullNorUndefined(this.state[valueKey])
              ? this.state[valueKey]
              : values[valueKey]
          };
        })
      )
    );
  }

  validateValues = () => {
    const { fields, submitAll } = this.props;

    let values = submitAll ? this.getAllCurrentValues() : {};

    let error = false;

    fields.forEach(({ field, mapFieldsToDB, validate, errorMessage }) => {
      if (mapFieldsToDB) {
        values[field] = undefined;
        //if mapFieldsToDB is used, the field value itself is unimportant and probably unused
        values = Object.assign(values, mapFieldsToDB(this.state));
      } else {
        if (notNullNorUndefined(this.state[field])) {
          values[field] = this.state[field];
        }
      }

      if (validate && validate(this.state[field]) !== true) {
        error = true;
        this.setState({ [field + "Error"]: errorMessage });
      }
    });

    return { values, error };
  };

  saveValues = () => {
    const { onComplete } = this.props;

    this.setState({ loading: true }, () => {
      Keyboard.dismiss();
      // NB: Before going to a new screen (on onComplete) keyboard should always be closed (assumption)

      const { values, error } = this.validateValues();

      if (error) {
        this.setState({ loading: false });
      } else {
        console.log("No error with values: ", values);

        this.props
          .mutate(values)
          .then(({ data }) => {
            //for api, state, dispatch support etc, the result should be taken based on mutationtype or so.

            this.setState({ loading: false }, () => {
              onComplete && onComplete(data, values);
              if (this.props.clearOnComplete) this.clearState();
            });
          })
          .catch(e => console.log("ERROR", e));
      }
    });
  };

  renderSaveButton() {
    const { completeButtonBackground } = this.props;

    const completeButton = this.props.completeButton
      ? this.props.completeButton
      : "Save";

    return (
      <View
        style={{
          height: 50,
          alignItems: "flex-end",
          justifyContent: "center",
          paddingRight: 20,
          backgroundColor: completeButtonBackground
            ? completeButtonBackground
            : "#ecf0f1"
        }}
      >
        {this.state.loading ? (
          <View style={{ marginLeft: 50 }}>
            <ActivityIndicator />
          </View>
        ) : (
          <Button title={completeButton} onPress={() => this.saveValues()} />
        )}
      </View>
    );
  }

  render() {
    const { fields, values, noScroll, noNavigationBelow } = this.props;

    const allFields = fields.map((field, index) => {
      return this.renderInput({
        field,
        value: values[field.field],

        /*
        is buggy with images
        
        field.mapFieldsToDB
          ? field.mapFieldsToDB(values)
          : 
          */
        key: `field-${index}`
      });
    });

    return noScroll ? (
      <View>
        {allFields}
        {this.renderSaveButton()}
      </View>
    ) : (
      <KAS
        noNavigationBelow={noNavigationBelow}
        footer={this.renderSaveButton()}
      >
        {allFields}
      </KAS>
    );
  }
}

export default DataForm;

==> react-native-data-forms-2.0.7.tgz: package/index.tsx <==
/**
 * #later
 * Currently, testing this repo is very annoying because I have to go through NPM publish and download it on the other project. Normally, it should be possible to use `npm link` (see `http://matthewsessions.com/2017/10/27/developing-rn-module.html`), but it doesn't work because the Metro Bundler `doesn't support symlinks`. (Error I got: `Expected path {path} to be relative to one of the project roots`). A solution would be using `Haul` but until recently that didn't work with `Expo` unless you ejected your project. However, in this issue, the below comment states he got it working: `https://github.com/callstack/haul/issues/463`. Try this approach!
 * 
 * - [ ] Try haul for Dunbar: see dunbar `haul.js`
 * - [ ] Symlink both projects at the same time.
 * - [ ] See if I can update on the fly
 * 
 * 
 * 
 * #marketing
- [ ] There must be other libraries exposing a range of input-type-components. 
      Would be great if it was easy to add this to it. #idea
- [ ] Copy docs into article and publish to medium, dev.to, LinkedIn and Karsens #later
- [ ] Wait for stars a week or so, after a week, reach out to tutorial writers and agencies like Spencer, 
      NativeBase, RN elements, Awesome react native, native.directory, slack groups, spectrum chats, and many more. #later
 *  */

import React from "react";

import {
  Text,
  ActivityIndicator,
  Platform,
  ScrollView,
  KeyboardAvoidingView,
  View,
  Keyboard
} from "react-native";

import { DataFormProps } from "./types";
import {
  removeDuplicates,
  notNullNorUndefined,
  mergeObjectArray
} from "./utils";

import { C } from "./constants";
import Button from "./button.component";

type DataFormState = {
  loading: boolean;
};

type KASProps = {
  noNavigationBelow: boolean;
  footer: React.ReactElement; // dunno
};

/**
 * KAS = KeyboardAvoidingScrollView
 * #todo: externalize to `leckr-util` or so
 */
class KAS extends React.Component<KASProps> {
  render() {
    const { noNavigationBelow, footer, children } = this.props;

    const offsetAdded = noNavigationBelow && Platform.OS === "ios" ? 50 : 0;
    const defaultOffset = Platform.OS === "ios" ? 20 : 75;
    const totalOffset = defaultOffset + offsetAdded;

    return (
      <KeyboardAvoidingView
        style={{
          height: "100%",
          backgroundColor: "#DDD"
        }}
        behavior="padding"
        keyboardVerticalOffset={totalOffset}
      >
        <ScrollView
          keyboardShouldPersistTaps={"handled"}
          style={{
            backgroundColor: C.BG_COLOR
          }}
        >
          {children}
        </ScrollView>
        {footer}
      </KeyboardAvoidingView>
    );
  }
}

class DataForm extends React.Component<DataFormProps, DataFormState> {
  constructor(props: DataFormProps) {
    super(props);
    this.saveValues = this.saveValues.bind(this); //to give props
    this.state = { loading: false };
  }

  renderInput = ({ field, value, key }) => {
    const { inputTypes, FieldComponent } = this.props;

    const isHidden = !!(
      field.hidden && field.hidden(this.getAllCurrentValues())
    );

    const inputProps = {
      state: this.state,
      setFormState: newState => this.setState(newState),
      value,
      ...field
    };

    const InputClass = field.type
      ? inputTypes[field.type]
      : inputTypes[Object.keys(inputTypes)[0]];

    if (InputClass && !isHidden) {
      const inputField = <InputClass {...inputProps} />;
      return FieldComponent({
        inputField,
        inputProps: { ...field },
        state: this.state,
        key
      });
    }

    return null;
  };

  clearState = () => {
    const { fields } = this.props;
    const allFields = fields.reduce(
      (result, f) => ({ ...result, [f.field]: undefined }),
      {}
    );

    this.setState(allFields);
  };

  getAllCurrentValues() {
    const { fields, values } = this.props;

    const valueKeysFromFields = fields.map(field => field.field);
    const valueKeysFromValues = Object.keys(values);
    const reallyAllValueKeys = removeDuplicates(
      valueKeysFromFields.concat(valueKeysFromValues)
    );

    return (
      values &&
      mergeObjectArray(
        reallyAllValueKeys.map(valueKey => {
          return {
            [valueKey]: notNullNorUndefined(this.state[valueKey])
              ? this.state[valueKey]
              : values[valueKey]
          };
        })
      )
    );
  }

  validateValues = () => {
    const { fields, submitAll } = this.props;

    let values = submitAll ? this.getAllCurrentValues() : {};

    let error = false;

    fields.forEach(({ field, mapFieldsToDB, validate, errorMessage }) => {
      if (mapFieldsToDB) {
        values[field] = undefined;
        //if mapFieldsToDB is used, the field value itself is unimportant and probably unused
        values = Object.assign(values, mapFieldsToDB(this.state[field]));
      } else {
        if (notNullNorUndefined(this.state[field])) {
          values[field] = this.state[field];
        }
      }

      if (validate && validate(this.state[field]) !== true) {
        error = true;
        this.setState({ [field + "Error"]: errorMessage });
      }
    });

    return { values, error };
  };

  saveValues = () => {
    const { onComplete } = this.props;

    this.setState({ loading: true }, () => {
      Keyboard.dismiss();
      // NB: Before going to a new screen (on onComplete) keyboard should always be closed (assumption)

      const { values, error } = this.validateValues();

      if (error) {
        this.setState({ loading: false });
      } else {
        console.log("No error with values: ", values);

        this.props
          .mutate(values)
          .then(({ data }) => {
            //for api, state, dispatch support etc, the result should be taken based on mutationtype or so.

            this.setState({ loading: false }, () => {
              onComplete && onComplete(data, values);
              if (this.props.clearOnComplete) this.clearState();
            });
          })
          .catch(e => console.log("ERROR", e));
      }
    });
  };

  renderSaveButton() {
    const { completeButtonBackground } = this.props;

    const completeButton = this.props.completeButton
      ? this.props.completeButton
      : "Save";

    return (
      <View
        style={{
          height: 50,
          alignItems: "flex-end",
          justifyContent: "center",
          paddingRight: 20,
          backgroundColor: completeButtonBackground
            ? completeButtonBackground
            : "#ecf0f1"
        }}
      >
        {this.state.loading ? (
          <View style={{ marginLeft: 50 }}>
            <ActivityIndicator />
          </View>
        ) : (
          <Button title={completeButton} onPress={() => this.saveValues()} />
        )}
      </View>
    );
  }

  render() {
    const { fields, values, noScroll, noNavigationBelow } = this.props;

    const allFields = fields.map((field, index) => {
      return this.renderInput({
        field,
        value: values[field.field],

        /*
        is buggy with images
        
        field.mapFieldsToDB
          ? field.mapFieldsToDB(values)
          : 
          */
        key: `field-${index}`
      });
    });

    return noScroll ? (
      <View>
        {allFields}
        {this.renderSaveButton()}
      </View>
    ) : (
      <KAS
        noNavigationBelow={noNavigationBelow}
        footer={this.renderSaveButton()}
      >
        {allFields}
      </KAS>
    );
  }
}

export default DataForm;
